package com.muyu.filter;

import com.muyu.common.constant.JwtConstants;
import com.muyu.common.constant.TokenConstants;
import com.muyu.common.redis.service.RedisService;
import com.muyu.common.utils.JwtUtils;
import com.muyu.common.utils.StringUtils;
import com.muyu.config.IgnoreWhiteConfig;
import com.muyu.utils.GatewayUtils;
import io.jsonwebtoken.Claims;
import lombok.AllArgsConstructor;
import org.springframework.cloud.context.config.annotation.RefreshScope;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.http.HttpHeaders;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.net.URI;
import java.nio.charset.StandardCharsets;
import java.util.concurrent.TimeUnit;

/**
 * @Author: WangXin
 * @date: 2024/7/23
 * @Description: 权限过滤器
 * @Version: 1.0.0
 */
@Component
@AllArgsConstructor
public class AuthFilter implements GlobalFilter, Ordered {

    private final IgnoreWhiteConfig ignoreWhiteConfig;

    private final RedisService redisService;

    @Override
    public Mono<Void> filter (ServerWebExchange exchange, GatewayFilterChain chain) {
        //获取请求路径
        ServerHttpRequest request = exchange.getRequest();
        URI uri = request.getURI();
        String path = uri.getPath();
        //判断请求路径与白名单中的请求路径进行比较
        if (StringUtils.matches(path, ignoreWhiteConfig.getWhites())){
            // 白名单里面存在 放行
            return chain.filter(exchange);
        }
        // 拿去请求头中的token
        HttpHeaders requestHeaders = request.getHeaders();
        String token = requestHeaders.getFirst(TokenConstants.TOKEN);
        // 判断token是否为空
        if (StringUtils.isEmpty(token)){

            return GatewayUtils.errorResponse(exchange, "token不合法");
        }
        //解析token
        Claims claims = JwtUtils.parseToken(token);
        //判断token格式
        if (claims == null){
            return GatewayUtils.errorResponse(exchange, "token不合法");
        }
        //拿出token中的userKey
        String userKey = JwtUtils.getUserKey(claims);
        //判断redis中的token是否过期
        if (!redisService.hasKey(TokenConstants.LOGIN_TOKEN_KEY + userKey)){
            return GatewayUtils.errorResponse(exchange, "token已过期");
        }

        long time = redisService.redisTemplate.getExpire(TokenConstants.LOGIN_TOKEN_KEY + userKey,TimeUnit.SECONDS);

        if (time>=TokenConstants.IS_OVERDUE_TIME){
            //给token续期
            redisService.expire(TokenConstants.LOGIN_TOKEN_KEY + userKey,TokenConstants.REFRESH_TIME, TimeUnit.MINUTES);
        }



        ServerHttpRequest.Builder mutate = request.mutate();
        //添加请求头 userKey userId userName
        GatewayUtils.addHeader(mutate, JwtConstants.USER_KEY, userKey);
        GatewayUtils.addHeader(mutate, JwtConstants.DETAILS_USER_ID, JwtUtils.getUserId(claims));
        String userName = JwtUtils.getUserName(claims);
        //判断 userName 是否是中文
        if (userName.matches(".*[\\u4E00-\\u9FA5]+.*")){
            //是 中文转换格式（请求头只支持 A表中的内容）
            userName = new String(userName.getBytes(StandardCharsets.UTF_8), StandardCharsets.ISO_8859_1);
        }

        GatewayUtils.addHeader(mutate, JwtConstants.DETAILS_USERNAME,userName );
        //放行
        return chain.filter(exchange);
    }

    /**
     * Get the order value of this object.
     * <p>Higher values are interpreted as lower priority. As a consequence,
     * the object with the lowest value has the highest priority (somewhat
     * analogous to Servlet {@code load-on-startup} values).
     * <p>Same order values will result in arbitrary sort positions for the
     * affected objects.
     *
     * @return the order value
     *
     * @see #HIGHEST_PRECEDENCE
     * @see #LOWEST_PRECEDENCE
     *
     * 拦截器的权重
     */
    @Override
    public int getOrder () {
        return 0;
    }
}
